/*==================================================================
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corp. 2003.  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 *===============================================================*/

package contactlist.struts.persistence;

import java.util.List;

import contactlist.forms.Contact;
import contactlist.struts.utilities.AIMException;

/**
 * Abstract class defines persistence broker interface
 * 
 * Creation date: (11/30/2003 10:00:00 PM)
 * @author: Tim Hanis
 */
public abstract class ContactListBroker {
	
	//  Persistence broker
	protected static ContactListBroker brokerInstance = null;
	
	//  Get instance singleton
	public static ContactListBroker getInstance() {
		return brokerInstance;
	}
	
	public abstract void populate();
	
	/**
	 * Get the Contact information given a contact number
	 * @param owner java.lang.String
	 * @param contactOID java.lang.String
	 * @return Contact
	 */
	public abstract Contact getContact(String owner, String contactOID) throws AIMException;
	
	
	/**
	 * Create a Contact
	 * @param Contact
	 */
	public abstract void saveContact(Contact contact) throws AIMException;
	
	
	/**
	 * Delete a Contact
	 * @param owner java.lang.String
	 * @param contactOID java.lang.String
	 */
	public abstract void deleteContact(String owner, String oid) throws AIMException;
	
	
	/**
	 * Get the contact list
	 * @param owner java.lang.String
	 * @return List of Contact objects
	 */
	public abstract List getContactList(String owner) throws AIMException;

}