package com.ibm.faces.sdo;

import javax.faces.el.PropertyNotFoundException;
import javax.faces.el.PropertyResolver;

import com.ibm.websphere.sdo.DataObjectAccessBean;
import com.ibm.websphere.sdo.access.util.TypeCoercionUtil;

import commonj.sdo.DataObject;
import commonj.sdo.Property;


public class SDOPropertyResolver extends PropertyResolver {

	PropertyResolver oldPropResolver;

	public SDOPropertyResolver(PropertyResolver propResolver) {
		oldPropResolver = propResolver;
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getValue(java.lang.Object, java.lang.Object)
	 */
	// TODO: Check that this is the correct behavior - the method has been changed to use
	//       and Object as thesecond param from a String.
	public Object getValue(Object base, Object property)
		throws PropertyNotFoundException {

//		if ((base == null) || (property == null)) {
//			String className =
//				base == null ? "null" : base.getClass().getName();
//			throw new PropertyNotFoundException(
//				"Error getting property '"
//					+ property
//					+ "' in bean of type "
//					+ className);
//		}

		try {
			if(base instanceof DataObject){
				if(property != null){
					return (((DataObject) base).get(property.toString()));
				}
			} 
			else if (base instanceof DataObjectAccessBean) {
				return (((DataObjectAccessBean) base).get(property));
			}
			
		} catch (NoClassDefFoundError ncdfe) {
			// wdo jars are not available
		}

		if (oldPropResolver != null) {
			return oldPropResolver.getValue(base, property);
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getValue(java.lang.Object, int)
	 */
	public Object getValue(Object base, int index)
		throws PropertyNotFoundException {

		try {
			if(base instanceof DataObject){
				return (((DataObject) base).get(index));
			} 
		} catch (NoClassDefFoundError ncdfe) {
			// wdo jars are not available
		}
		
		if (oldPropResolver != null) {
			return oldPropResolver.getValue(base, index);
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#setValue(java.lang.Object, java.lang.Object, java.lang.Object)
	 */
	// TODO: Check that this is the correct behavior - the method has been changed to use
	//       and Object as thesecond param from a String.
	public void setValue(Object base, Object property, Object value)
		throws PropertyNotFoundException {

		if ((base == null) || (property == null)) {
			String className =
				base == null ? "null" : base.getClass().getName();
			throw new PropertyNotFoundException(
				"Error setting property '"
					+ property
					+ "' in bean of type "
					+ className);
		}

		try {
			if(base instanceof DataObject){
				if(property != null){
					((DataObject) base).set(property.toString(),
							TypeCoercionUtil.coerceType(getType(base, property)
									.getName(), value));
					return;
				}
			} 
			else if (base instanceof DataObjectAccessBean) {
				((DataObjectAccessBean) base).put(property, value);
				return;
			}
		} catch (NoClassDefFoundError ncdfe) {
			// wdo jars are not available
		}

		if (oldPropResolver != null) {
			oldPropResolver.setValue(base, property, value);
			return;
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#setValue(java.lang.Object, int, java.lang.Object)
	 */
	public void setValue(Object base, int index, Object value)
		throws PropertyNotFoundException {

		try {
			if(base instanceof DataObject){
				((DataObject) base).set(index,
						TypeCoercionUtil.coerceType(getType(base, index)
								.getName(), value));
				return;
			} 
		} catch (NoClassDefFoundError ncdfe) {
			// wdo jars are not available
		}
		
		if (oldPropResolver != null) {
			oldPropResolver.setValue(base, index, value);
			return;
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#isReadOnly(java.lang.Object, java.lang.Object)
	 */
	// TODO: Check that this is the correct behavior - the method has been changed to use
	//       and Object as thesecond param from a String.
	public boolean isReadOnly(Object base, Object property)
		throws PropertyNotFoundException {

		if ((base == null) || (property == null)) {
			String className =
				base == null ? "null" : base.getClass().getName();
			throw new PropertyNotFoundException(
				"Error testing property '"
					+ property
					+ "' in bean of type "
					+ className);
		}

		try {
			if (base instanceof DataObjectAccessBean) {
				if (property instanceof String) {
					return ((DataObjectAccessBean) base).isReadOnly(
						(String) property);
				} else {
					String className =
						base == null ? "null" : base.getClass().getName();
					throw new PropertyNotFoundException(
						"Property '"
							+ property
							+ "' is not a String in bean of type "
							+ className);
				}
			}
		} catch (NoClassDefFoundError ncdfe) {
			// wdo jars are not available
		} catch (Exception e) {
			//TODO - behavior on error? cannot directly catch MediatorException (causes ncdfe if no wdo)
			throw new PropertyNotFoundException(e);
		}

		if (oldPropResolver != null) {
			return oldPropResolver.isReadOnly(base, property);
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#isReadOnly(java.lang.Object, int)
	 */
	public boolean isReadOnly(Object base, int index)
		throws PropertyNotFoundException {

		if (base == null) {
			throw new NullPointerException();
		}
		try {
			if (!(base instanceof DataObjectAccessBean)) {
				return oldPropResolver.isReadOnly(base, index);
			}
		} catch (NoClassDefFoundError ncdfe) {
			// wdo jars are not available
			return oldPropResolver.isReadOnly(base, index);
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getType(java.lang.Object, java.lang.Object)
	 */
	// TODO: Check that this is the correct behavior - the method has been changed to use
	//       and Object as thesecond param from a String.
	public Class getType(Object base, Object property)
		throws PropertyNotFoundException {

		if ((base == null) || (property == null)) {
			String className =
				base == null ? "null" : base.getClass().getName();
			throw new PropertyNotFoundException(
				"Error testing property '"
					+ property
					+ "' in bean of type "
					+ className);
		}

		try {
			if(base instanceof DataObject){
				if(property != null){
					Property prop = 
						((DataObject) base).getType().getProperty(property.toString());
					if(prop != null){
						return prop.getType().getInstanceClass();
					}
				}
			}
			else if (base instanceof DataObjectAccessBean) {
				if (property instanceof String) {
					return ((DataObjectAccessBean) base).getType(
						(String) property);
				} else {

					String className =
						base == null ? "null" : base.getClass().getName();
					throw new PropertyNotFoundException(
						"Error testing property '"
							+ property
							+ "' in bean of type "
							+ className);
				}
			}
		} catch (NoClassDefFoundError ncdfe) {
			// wdo jars are not available
		} catch (Exception e) {
			//TODO - behavior on error? cannot directly catch MediatorException (causes ncdfe if no wdo)
			throw new PropertyNotFoundException(e);
		}

		if (oldPropResolver != null) {
			return oldPropResolver.getType(base, property);
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getType(java.lang.Object, int)
	 */
	public Class getType(Object base, int index)
		throws PropertyNotFoundException {

		if (base == null) {
			throw new NullPointerException();
		}
		try {
			if(base instanceof DataObject){
				Property prop = 
					(Property)	
					((DataObject) base).getType().getProperties().get(index);
				return prop.getType().getInstanceClass();
			}
			if (!(base instanceof DataObjectAccessBean)) {
				return oldPropResolver.getType(base, index);
			}
		} catch (NoClassDefFoundError ncdfe) {
			// wdo jars are not available
			return oldPropResolver.getType(base, index);
		}
		throw new PropertyNotFoundException();
	}

}
