package com.ibm.websphere.samples.bank.ejb;

import java.util.ListResourceBundle;
import java.util.ResourceBundle;

import javax.ejb.CreateException;
import javax.naming.InitialContext;

import com.ibm.websphere.samples.bank.exception.InsufficientFundsException;

/**
 * @ws.sdo.value-object
 * 	name="Account"
 * 	read-only="false"
 * Bean implementation class for Enterprise Bean: Account
 */ 
public abstract class AccountBean implements javax.ejb.EntityBean {
	private javax.ejb.EntityContext myEntityCtx;
	
	private ListResourceBundle bundle = (ListResourceBundle) ResourceBundle.getBundle("com.ibm.websphere.samples.bank.ejb.AccountResourceBundle");
	private final static String CUSTOMER_JNDI_NAME = "java:comp/env/ejb/Customer";


	/**
	* Adds the specified amount to the account balance and returns the new balance.
	* @return float The new balance in the account after the method executes.
	* @param amount float The amount to add to the account balance.
	*/
	public float add(float amount) {
	    setBalance(getBalance() + amount);
	    return getBalance();
	}
	/**
	* Subtracts the specified amount from the account balance and returns the new balance.
	* @return float The new balance in the account after the method executes.
	* @param amount float The amount to subtract from the account balance.
	* @exception com.ibm..bank.ejb.InsufficientFundsException The account contained 
	* less than the amount requested for subtraction.
	*/
	public float subtract(float amount) throws InsufficientFundsException {
	    if (getBalance() < amount) {
	        throw new InsufficientFundsException(bundle.getString("insufficientFunds"));
	    }
	    setBalance(getBalance() - amount);
	    return getBalance();
	}

	/**
	 * ejbCreate
	 */
	public com.ibm.websphere.samples.bank.ejb.AccountKey ejbCreate(int accountNumber, int type, float balance, long customerNumber) throws javax.ejb.CreateException {
	    setAccountNumber(accountNumber);
	    setAccountType(type);
	    setBalance(balance);
	    return null;
	}


	/**
	 * ejbPostCreate adds this newly created Account to the collection of Accounts owned by the customer
	 * pointed to by customerNumber
	 */
	public void ejbPostCreate(int accountNumber, int type, float initialBalance, long customerNumber) throws javax.ejb.CreateException { //**** added customer number
	   try {
	      AccountLocal thisAccount = (AccountLocal) getEntityContext().getEJBLocalObject(); // get this object's local interface
	      InitialContext initCtx = new InitialContext();
	      CustomerLocalHome customerHome = (CustomerLocalHome) initCtx.lookup(CUSTOMER_JNDI_NAME);

	      CustomerLocal customer = customerHome.findByPrimaryKey(new CustomerKey(customerNumber)); //get the customer
	      customer.addAccount(thisAccount); //add this account
	   } catch (Exception e) {
	      e.printStackTrace(); //for debugging
	      throw new CreateException(e.getMessage()); //throws create exception - something went wrong
	   }
	}

	
	/**
	 * setEntityContext
	 */
	public void setEntityContext(javax.ejb.EntityContext ctx) {
		myEntityCtx = ctx;
	}
	/**
	 * getEntityContext
	 */
	public javax.ejb.EntityContext getEntityContext() {
		return myEntityCtx;
	}
	/**
	 * unsetEntityContext
	 */
	public void unsetEntityContext() {
		myEntityCtx = null;
	}
	/**
	 * ejbCreate
	 */
	public com.ibm.websphere.samples.bank.ejb.AccountKey ejbCreate(
		int accountNumber) throws javax.ejb.CreateException {
		setAccountNumber(accountNumber);
		return null;
	}			/**
	 * ejbPostCreate
	 */
	public void ejbPostCreate(int accountNumber)
		throws javax.ejb.CreateException {
	}/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbLoad
	 */
	public void ejbLoad() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() throws javax.ejb.RemoveException {
	}
	/**
	 * ejbStore
	 */
	public void ejbStore() {
	}
	/**
	 * Get accessor for persistent attribute: accountNumber
	 */
	public abstract int getAccountNumber();
	/**
	 * Set accessor for persistent attribute: accountNumber
	 */
	public abstract void setAccountNumber(int newAccountNumber);
	/**
	 * Get accessor for persistent attribute: accountType
	 */
	public abstract int getAccountType();
	/**
	 * Set accessor for persistent attribute: accountType
	 */
	public abstract void setAccountType(int newAccountType);
	/**
	 * Get accessor for persistent attribute: balance
	 */
	public abstract float getBalance();
	/**
	 * Set accessor for persistent attribute: balance
	 */
	public abstract void setBalance(float newBalance);
}