package com.ibm.websphere.samples.bank.ejb;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import com.ibm.websphere.samples.bank.util.AccountData;

/**
 * @ws.sbf.session-facade
 * 	name="CustomerFacade"
 * 	value-objects="Customer"
 * @ws.sdo.value-object
 * 	name="Customer"
 * 	read-only="false"
 * Bean implementation class for Enterprise Bean: Customer
 */
public abstract class CustomerBean implements javax.ejb.EntityBean {
	private javax.ejb.EntityContext myEntityCtx;
	
	/*.....
	  Allows adding an account from client code to the collection of owned accounts 
	  without letting the client manipulate the collection
	  ......*/

	public void addAccount(AccountLocal anAccount) { //added for  CMR
	    if (anAccount != null)
	        getAccounts().add(anAccount);
	}


	/*.....
	  Exposes the collection of Owned Account Numbers  to the client(numbers only)
	  ......*/
	public Vector getOwnedAccountNumbers() { //returns all the acct numbers to the client - for CMR verification
	    Vector allAccountNumbers = new Vector();
	    Collection allAccounts = getAccounts();
	    Iterator iterator = allAccounts.iterator();
	    while (iterator.hasNext()) {
	        AccountLocal account = (AccountLocal) iterator.next();
	        Long acctNumber = new Long(account.getAccountNumber());
	        allAccountNumbers.add(acctNumber);
	    }
	    return allAccountNumbers;

	}
	
	/*.....................................................
    Exposes the result of cmr relationship to a client
    @return collection of Account object
    .....................................................*/
  public Collection getAccountsList() {

      ArrayList list = null;
      try {
          Collection allAccounts = getAccounts();

          Iterator it = allAccounts.iterator();
          while (it.hasNext()) {
              if (list == null)
                  list = new ArrayList();

              AccountLocal account = (AccountLocal) it.next();
              long acctNumber = (new Long(account.getAccountNumber())).longValue();
              int acctType = account.getAccountType();
              float balance = account.getBalance();
              AccountData data = new AccountData(acctNumber, balance, acctType);
              list.add(data);
          }

      } catch (Exception e) {
          e.printStackTrace();
      }
      return list;
  }

	/**
	 * ejbCreate
	 */
	public com.ibm.websphere.samples.bank.ejb.CustomerKey ejbCreate(CustomerKey customerKey, String name, String lname, String taxID) throws javax.ejb.CreateException {
	    setCustomerNumber(customerKey.getCustomerNumber());
	    setFirstName(name);
	    setLastName(lname);
	    setTaxID(taxID);
	    return null;
	}


	/**
	     * ejbPostCreate
	     */
	public void ejbPostCreate(CustomerKey customerKey, String name, String lname, String taxID) throws javax.ejb.CreateException {
	}

	/**
	 * setEntityContext
	 */
	public void setEntityContext(javax.ejb.EntityContext ctx) {
		myEntityCtx = ctx;
	}
	/**
	 * getEntityContext
	 */
	public javax.ejb.EntityContext getEntityContext() {
		return myEntityCtx;
	}
	/**
	 * unsetEntityContext
	 */
	public void unsetEntityContext() {
		myEntityCtx = null;
	}
	/**
	 * ejbCreate
	 */
	public com.ibm.websphere.samples.bank.ejb.CustomerKey ejbCreate(
		long customerNumber) throws javax.ejb.CreateException {
		setCustomerNumber(customerNumber);
		return null;
	}			/**
	 * ejbPostCreate
	 */
	public void ejbPostCreate(long customerNumber)
		throws javax.ejb.CreateException {
	}/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbLoad
	 */
	public void ejbLoad() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() throws javax.ejb.RemoveException {
	}
	/**
	 * ejbStore
	 */
	public void ejbStore() {
	}
	/**
	 * @ws.sdo.value-object
	 * 	match="Customer"
	 * Get accessor for persistent attribute: customerNumber
	 */
	public abstract long getCustomerNumber();
	/**
	 * Set accessor for persistent attribute: customerNumber
	 */
	public abstract void setCustomerNumber(long newCustomerNumber);
	/**
	 * @ws.sdo.value-object
	 * 	match="Customer"
	 * Get accessor for persistent attribute: lastName
	 */
	public abstract java.lang.String getLastName();
	/**
	 * Set accessor for persistent attribute: lastName
	 */
	public abstract void setLastName(java.lang.String newLastName);
	/**
	 * @ws.sdo.value-object
	 * 	match="Customer"
	 * Get accessor for persistent attribute: firstName
	 */
	public abstract java.lang.String getFirstName();
	/**
	 * Set accessor for persistent attribute: firstName
	 */
	public abstract void setFirstName(java.lang.String newFirstName);
	/**
	 * @ws.sdo.value-object
	 * 	match="Customer"
	 * Get accessor for persistent attribute: taxID
	 */
	public abstract java.lang.String getTaxID();
	/**
	 * Set accessor for persistent attribute: taxID
	 */
	public abstract void setTaxID(java.lang.String newTaxID);
	/**
	 * @ws.sdo.value-object
	 * 	match="Customer"
	 * 	contained="true"
	 * This method was generated for supporting the relationship role named accounts.
	 * It will be deleted/edited when the relationship is deleted/edited.
	 */
	public abstract java.util.Collection getAccounts();	/**
	 * This method was generated for supporting the relationship role named accounts.
	 * It will be deleted/edited when the relationship is deleted/edited.
	 */
	public abstract void setAccounts(java.util.Collection anAccounts);	
	
	public abstract java.util.Collection ejbSelectAccountsByBalance(long customerNumber, float balance) throws javax.ejb.FinderException;
	/**.....................................................
     * Exposes the result of the ejbSelect method to a client
     @return a collection of AccountNumber as Integer object
     .....................................................*/
public Vector getAccountsByBalance(float minimumBalance) {
   final String methodName = "getAccountByBalance";

   try {
       Vector v = new Vector();
       Collection c = ejbSelectAccountsByBalance(getCustomerNumber(), minimumBalance);
       Iterator iterator = c.iterator();
       while (iterator.hasNext()) {

           v.add(iterator.next());
       }
       return v;
   } catch (javax.ejb.FinderException e) {
       e.printStackTrace();
   }
   return null;
}
/*.....................................................
 Exposes the result of the ejbSelect method to a client
 .....................................................*/
public int getAccountsCount() {
   final String methodName = "getAccountsCount";

   int c = 0;
   try {
       c = ejbSelectAccounts(getCustomerNumber());

   } catch (javax.ejb.FinderException e) {
       e.printStackTrace();
   } catch (Exception e) {
       e.printStackTrace();
   }
   return c;
}

	public abstract int ejbSelectAccounts(long customerNumber) throws javax.ejb.FinderException;
}