//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

// General EJB.
import java.io.IOException;
import java.io.Serializable;
import java.util.ResourceBundle;

import javax.ejb.DuplicateKeyException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.AccountKey;
import com.ibm.websphere.samples.bank.ejb.AccountLocal;
import com.ibm.websphere.samples.bank.ejb.AccountLocalHome;

import java.util.logging.*;

/**
 * Utility servlet to create an Account entity bean.
 **/
public class CreateAccount extends HttpServlet implements Serializable {

    private ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.websphere.samples.bank.web.ClientResourceBundle");

    // Strings for HTML output - US English defaults shown.
    static String createdaccount = "Created account:";
    static String createdtype = "Type:";
    static String createdbalance = "Balance:";
    static String failureexists = "Create failure - account already exists:";
    static String failureinternal = "Internal failure - could not create account:";
    static String failureaccount = "Invalid account format:";
    static String failurebalance = "Invalid balance format:";
    static String initError = "Error initializing Servlet.";

    // EJB access.
    private AccountLocalHome accountHome = null;
    private final static String ACCOUNT_JNDI_NAME = "java:comp/env/ejb/Account";

    static final String className = CreateAccount.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
     * Read in internationalization strings when servlet is loaded. <p>
     * Create AccountHome when servlet is loaded. <p>
     * @param config Configuration information passed by the servlet engine.
     * @exception javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
     */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";
        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);
        // Get national language text for HTML response.
        try {
            // Get NLS strings from an external resource bundle identified by 
            // "com.ibm.websphere.samples.bank.web.ClientResourceBundle", to override 
            // US English defaults.

            createdaccount = bundle.getString("createdaccount");
            createdtype = bundle.getString("createdtype");
            createdbalance = bundle.getString("createdbalance");
            failureexists = bundle.getString("failureexists");
            failureinternal = bundle.getString("failureinternal");
            failureaccount = bundle.getString("failureaccount");
            failurebalance = bundle.getString("failurebalance");
            initError = bundle.getString("initError");

        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, "init resource bundle exception: " + e.getMessage());
            throw new ServletException(e.getMessage());
        }

        try {
            Context ctx = new InitialContext();
            Object homeObject = ctx.lookup(ACCOUNT_JNDI_NAME);

            accountHome = (AccountLocalHome) homeObject;

        }
        // Determine cause of failure.
        catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("failureinit") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("failuregenexp") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }
    }

    public void doPost(HttpServletRequest req, HttpServletResponse res) throws IOException, ServletException {
        performTask(req, res);
    }

    public void doGet(HttpServletRequest req, HttpServletResponse res) throws IOException, ServletException {
        performTask(req, res);
    }

    /**
     * Respond to user request to create an Account entity bean. <p>
     * @param req Used to get information about the user request - an account number, etc.
     * @param res Used to pass response back to the user - an output stream for the returned HTML, etc.
     * @exception javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
     * @exception java.io.IOException Signals that an I/O exception of some sort has occurred.
     */
    public void performTask(HttpServletRequest req, HttpServletResponse res) throws ServletException {
        final String methodName = "performTask";

        // --- Read and validate user input, initialize. ---
        // Error flags.      
        boolean accountFlag = true;
        boolean balanceFlag = true;
        boolean customerFlag = true; //added for CMR
        boolean inputFlag = false;
        boolean createFlag = true;
        boolean duplicateFlag = false;

        // Datatypes used to create new account.
        AccountKey key;
        int typeAcct = 0;
        String typeString = "0";
        float initialBalance = 0;

        try {

            // Read input parameters from HTML form.
            String[] accountArray = req.getParameterValues("account");
            String[] typeArray = req.getParameterValues("type");
            String[] balanceArray = req.getParameterValues("balance");
            String[] customerNumberArray = req.getParameterValues("customerNumber"); //added for CMR

            // Convert input parameters to needed datatypes for new account EJB.
            // (account)
            int accountNumber = 0;
            try {
                accountNumber = Integer.parseInt(accountArray[0]);
            } catch (Exception e) {
                accountFlag = false;
            }

            long customerNumber = 0;
            try {
                customerNumber = Long.parseLong(customerNumberArray[0]);
            } catch (Exception e) {
                customerFlag = false;
            }

            // (type)
            if (typeArray[0].equals("1")) {
                typeAcct = 1; // Savings account.
                typeString = "savings";
            } else if (typeArray[0].equals("2")) {
                typeAcct = 2; // Checking account
                typeString = "checking";
            }

            // (balance)
            try {
                initialBalance = (Float.valueOf(balanceArray[0])).floatValue();
            } catch (Exception e) {
                balanceFlag = false;
            }

            // --- If input parameters are good, try to create account EJB. ---
            if (accountFlag && balanceFlag && customerFlag) {
                inputFlag = true;
                try {
                    // Create the EJB.
                    AccountLocal account = accountHome.create(accountNumber, typeAcct, initialBalance, customerNumber);
                }
                // Determine cause of failure.
                catch (DuplicateKeyException e) {
                    createFlag = false;
                    duplicateFlag = true;
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("failuredupkey") + e.getMessage());
                    e.printStackTrace();
                } catch (Exception e) {
                    createFlag = false;
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("failuregenexp") + e.getMessage());
                    e.printStackTrace();
                }
            }

            // --- Prepare message to accompany response. ---
            String messageLine = "";
            if (inputFlag) {
                // If you are here, the client input is good.
                if (createFlag) {
                    // New account EJB was created.
                    messageLine = createdaccount + " " + accountArray[0] + ", " + createdtype + " " + typeString + ", " + createdbalance + " " + balanceArray[0];
                } else if (duplicateFlag) {
                    // Account with same key already exists.
                    messageLine = failureexists + " " + accountArray[0];
                } else {
                    // Other reason for failure.
                    messageLine = failureinternal + " " + accountArray[0];
                }
            } else {
                // If you are here, something was wrong with the client input.
                String separator = "";
                if (!accountFlag) {
                    messageLine = failureaccount + " " + accountArray[0];
                    separator = ", ";
                }
                if (!balanceFlag) {
                    messageLine = messageLine + separator + failurebalance + " " + balanceArray[0];
                }
            }

            JSPBean jspbean = new JSPBean();
            jspbean.setMessageLine(messageLine);
            javax.servlet.http.HttpSession session = req.getSession(true);
            session.setAttribute("JSPBean", jspbean);
            RequestDispatcher rd = getServletContext().getRequestDispatcher("CreateAccountJSP.jsp");
            rd.forward(req, res);
        } catch (Throwable theException) {
            // uncomment the following line when unexpected exceptions are occuring to aid in debugging the problem
            // theException.printStackTrace();
            throw new ServletException(theException);
        }

    }
}
