//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

import java.io.IOException;
import java.io.Serializable;
import java.util.PropertyResourceBundle;

import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.AccountLocal;
import com.ibm.websphere.samples.bank.ejb.AccountLocalHome;

import java.util.logging.*;

public class InterestServlet extends HttpServlet implements Serializable {

    // EJB access.
    private AccountLocalHome accountHome = null;
    private static final String PROPFILE = "/WEB-INF/properties/bank_messages";
    PropertyResourceBundle bundle = (PropertyResourceBundle) PropertyResourceBundle.getBundle(PROPFILE);
    private final static String ACCOUNT_JNDI_NAME = "java:comp/env/ejb/Account";

    static final String className = InterestServlet.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
      * @see javax.servlet.GenericServlet#init (ServletConfig)
      */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";
        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);

        try {
            javax.naming.Context ctx = new javax.naming.InitialContext();
            accountHome = (AccountLocalHome) ctx.lookup(ACCOUNT_JNDI_NAME);
        } catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errNamingInit") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInitExpection") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }
    }
    /**
    * @see javax.servlet.http.HttpServlet#doGet (javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);
    }

    /**
    * @see javax.servlet.http.HttpServlet#doPost (javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);

    }

    /**
     * @param req
     * @param resp
     * @throws ServletException
     */
    public void performTask(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        final String methodName = "performTask";

        try {
            PageBean pgBean = new PageBean();
            java.util.ArrayList msg = new java.util.ArrayList();
            java.util.ArrayList errmsg = new java.util.ArrayList();
            javax.servlet.http.HttpSession session = req.getSession(true);
            HistoryBean history = (HistoryBean) session.getAttribute("HistoryBean"); 
            if (history == null) {
      	        history = new HistoryBean(); // creates the bean if there isn't one on the session
            }

            String account = req.getParameter("account");
            String interest = req.getParameter("interest");
            String createtimerbutton = req.getParameter("createTimer");
            String canceltimerbutton = req.getParameter("cancelTimer");

            int accountnumber = 0;
            double interestamount = 0;

            if (account == null || account.trim().length() == 0) {
                errmsg.add(bundle.getString("errAcctNum"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctNum"));
            } else {
                try {
                    accountnumber = Integer.parseInt(account);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errAcctNumValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errAcctNumValid"));
                }
            }
            if (interest == null || interest.trim().length() == 0) {
                errmsg.add(bundle.getString("errInterest"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInterest"));
            } else {
                try {
                    interestamount = Double.parseDouble(interest);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errInterestValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInterestValid"));
                }
            }

            //After page validation if there are no errors 
            if (errmsg.size() == 0) {
                if (createtimerbutton != null && createtimerbutton.length() > 0) {
                    try {
                        //They pushed the create timer button.
                        AccountLocal accountobj = accountHome.findByPrimaryKey(new com.ibm.websphere.samples.bank.ejb.AccountKey(accountnumber));
                        //accountobj.myCreateTimer(interestamount);
                        msg.add(bundle.getString("succInterestCreate"));
                        history.addMessage(bundle.getString("succInterestCreate")); // adds a message to the command history
                    } catch (Exception e) {
                        errmsg.add(bundle.getString("errInterestCreate"));
                        messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInterestCreate"));
                        e.printStackTrace();
                    }
                } else {
                    try {
                        //They pushed the cancel timer button
                        AccountLocal accountobj = accountHome.findByPrimaryKey(new com.ibm.websphere.samples.bank.ejb.AccountKey(accountnumber));
                        //accountobj.myCancelTimer();
                        msg.add(bundle.getString("succInterestCancel"));
                        history.addMessage(bundle.getString("succInterestCancel")); // adds a message to the command history
                    } catch (Exception e) {
                        errmsg.add(bundle.getString("errInterestCancel"));
                        messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInterestCancel"));
                        e.printStackTrace();
                    }
                }
            } //end of if

            pgBean.setMessageLines(msg);
            pgBean.setErrormessage(errmsg);
            req.setAttribute("PageBean", pgBean);
            session.setAttribute("HistoryBean", history);// stores it back into the session
            getServletContext().getRequestDispatcher("/jsp/timer.jsp").forward(req, resp);
        } catch (Throwable ex) {
            throw new ServletException(ex);
        }

    }

}
